/*----------------------------------------------------------------------*/
/*  Board - this object holds all the information about the board:
        the screen positions of the pattern and the players attempts at
        reproducing the pattern.
          - the methods are: 
              ClosestFace - finds the closest face to a screen coord
              NumCorrect - counts the # correct attempts 
              Draw - draws the board
                                                                        */
/*----------------------------------------------------------------------*/
/*             Jim Morey  -  morey@math.ubc.ca  -  Mar 14,1996          */
/*----------------------------------------------------------------------*/
  
import java.io.*;
import java.awt.*;
import java.lang.*;

/*----------------------------------------------------------------------*/

public class Board extends Object {
  private final static int BLANK=-1;

  public int nvert,vertex[][],face[],faceTry[],boundary[][],closestface;
  public int minRot;
  private double dir[][];
  private int dist[];
  private boolean level1;

  /* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - */
  Board(StreamTokenizer token,GroupGraph group,boolean level1_) throws IOException{
    level1 = level1_;
    boundary = new int[2][2];
    boundary[0][0] = Integer.MAX_VALUE;
    boundary[0][1] = Integer.MAX_VALUE;
    boundary[1][0] = Integer.MIN_VALUE;
    boundary[1][1] = Integer.MIN_VALUE;

    token.nextToken();
    int x0 = (int)token.nval;
    token.nextToken();
    int y0 = (int)token.nval;
    token.nextToken();
    int nvec = (int)token.nval;
    dir = new double[nvec][2];

    for (int i=0;i<nvec;i++){
      token.nextToken();
      dir[i][0]= (double)token.nval;
      token.nextToken();
      dir[i][1]= (double)token.nval;
    }
          
    token.nextToken();
    nvert = (int)token.nval;
    vertex = new int[nvert][2];
    face = new int[nvert];
    faceTry = new int[nvert];
    dist = new int[nvert];
    
    for (int i=0;i<nvert;i++){
      token.nextToken();
      face[i]= (int)token.nval;
      minRot += group.minRot[face[i]];
      faceTry[i] = BLANK;
      double tmpx = x0; 
      double tmpy = y0; 
      for (int j=0;j<nvec;j++) {
        token.nextToken();
        int tmp= (int)token.nval;
        tmpx += tmp * dir[j][0] ;
        tmpy += tmp * dir[j][1];
      }
      vertex[i][0] = (int)tmpx;
      vertex[i][1] = (int)tmpy;
      for (int j=0;j<2;j++) {
        if (vertex[i][j] <boundary[0][j]) boundary[0][j] = vertex[i][j];
        if (vertex[i][j] >boundary[1][j]) boundary[1][j] = vertex[i][j];
      } 
    }
  }

  /* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - */
  public boolean ClosestFace(int x, int y, int maxdist) {
    closestface = 0;
    for (int i=0;i<nvert;i++){
      dist[i] = (x-vertex[i][0])*(x - vertex[i][0]) + 
        (y-vertex[i][1])*(y - vertex[i][1]); 
      if (dist[i]<dist[closestface]) closestface = i;
    }

    return (dist[closestface]<maxdist);
  }

  /* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - */
  public void Blank() {
    for (int i=0;i<nvert;i++)
      faceTry[i] = BLANK;
  }

  /* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - */
  public int NumCorrect() {
    int count=0;
    for (int i=0;i<nvert;i++)
      if (face[i]==faceTry[i]) count++;

    return count;
  }

  /* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - */
  public void Draw(Graphics g,Solid solid,GroupGraph group,int x0,int y0,boolean initial){
  /* don't do this except at the beginning */
    Omatrix tmp = new Omatrix();
    tmp.Equal(solid.orient);
    for (int i=0;i<nvert;i++){
      if (initial) {
        /* -- this is a bit of a fudge */
        solid.orient = group.element[solid.polygons[group.blank[face[i]]][1]];
        solid.DropBlank(g, group.blank[face[i]],group.tilt,
          vertex[i][0]+x0,vertex[i][1]+y0,level1);
      }
      else { 
        solid.orient = group.element[face[i]];
        solid.DropFace(g,group.tilt, vertex[i][0]+x0,vertex[i][1]+y0,level1);
      }
    } 
    solid.orient.Equal(tmp);
  }

  /* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - */
  public void printCurrent(){
    System.out.println("in the *.config file after #---pattern.. replace the first column with these");
    for (int i=0;i<nvert;i++) System.out.println(faceTry[i]);
  }

}
