/*----------------------------------------------------------------------*/
/*  CopyCat -- the game where you have to be a copycat                   
            This is the foreman object -- it tells all the other objects 
        to get to work and then it waits for PlayArea to tell it that 
        something happened.
            The one parameter that it accepts is a *.config file that 
        is full of information including component spacing, board 
        pattern, type of solid and solid orientations... 
            CopyCat restarts if the user clicks in the Titlepanel.
            CopyCat dumps the current info about the board if 'c' is 
        pressed.  This is a hidden feature since it's not a very user
        friendly thing -- not for the average user.  But for someone 
        fairly computer literate they can use this information to 
        construct their own boards. 
                                                                        */
/*----------------------------------------------------------------------*/
/*             Jim Morey  -  morey@math.ubc.ca  -  Mar 10,1996          */
/*----------------------------------------------------------------------*/

import java.io.*;
import java.applet.*;
import java.awt.*;
import java.net.*;
import java.lang.*;
import PlayArea;
import Rotator;
import Title;
import Board;
import BoardBox;

/*----------------------------------------------------------------------*/
public class CopyCat extends Applet implements Runnable{
  private final static int TITLE_H=70,PADDING=10;

  private Thread kicker;
  private Panel infopanel,titlepanel;
  private Title scores;
  private Title subtitle;
  private Board board;
  private PlayArea playarea;
  private AudioClip purrr;
  private boolean level1,finished=false;
  private int rotations;

  /* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - */
  public void init() {
    setBackground(new Color(255,200,200));
    Color foreground = Color.lightGray;
    String config = getParameter("config");
    URL here = this.getDocumentBase();
    try {
      InputStream is = new URL(here,config).openStream();
      StreamTokenizer token = new StreamTokenizer(is);
      token.eolIsSignificant(false);
      token.commentChar('#');
   
      /* .. get CopyCat info .. */
      token.nextToken();
      String solidData = new String(token.sval);
      token.nextToken();
      String groupData = new String(token.sval);
      token.nextToken();
      int solidSize = (int)token.nval;

      token.nextToken();
      int nWords = (int)token.nval;
      token.nextToken();
      String quote = new String(token.sval);
      for (int i=1;i<nWords;i++){
        token.nextToken();
        quote = quote.concat(" "+token.sval);
      }
      token.nextToken();
      int W = (int)token.nval;
      token.nextToken();
      int H = (int)token.nval;
      token.nextToken();
      int winW = (int)token.nval;
      token.nextToken();
      int RotatorH = (int)token.nval;
      token.nextToken();
      level1 = (1==(int)token.nval);

      resize(W,H);

      /* .. get solid info .. */
      InputStream is_solid = new URL(here,solidData).openStream();
      Solid solid = new Solid(is_solid,solidSize,solidSize,10);
      is_solid.close();
      /* .. reread the file since I'm have troubles with clone .. */
      is_solid = new URL(here,solidData).openStream();
      Solid solid1 = new Solid(is_solid,solidSize,solidSize,10);
      is_solid.close();
      is_solid = new URL(here,solidData).openStream();
      Solid solid2 = new Solid(is_solid,solidSize,solidSize,10);
      is_solid.close();

      /* .. get group info .. */
      InputStream is_group = new URL(here,groupData).openStream();
      GroupGraph group = new GroupGraph(is_group);
      is_group.close();

      /* .. construct Title .. */
      titlepanel = new Panel();
      titlepanel.setBackground(foreground);
      titlepanel.setLayout(new BorderLayout());
      Title title = new Title(this,"CopyCat",W-PADDING,37);
      titlepanel.add("North",title);
      subtitle = new Title(this,quote,W-PADDING,21);
      subtitle.font = new Font("TimesRoman",Font.ITALIC,18);
      subtitle.centre();
      titlepanel.add("South",subtitle);
      add(titlepanel);
  
      /* .. construct PlayArea .. */
      board = new Board(token,group,level1);
      playarea = new PlayArea(this,board,solid1,group,token,
        winW,H-TITLE_H,level1);
      add(playarea);

      /* .. construct infopanel -- a rotating solid and the board .. */
      infopanel = new Panel();
      infopanel.setBackground(foreground);
      infopanel.setLayout(new BorderLayout());
      BoardBox boardbox = new BoardBox(board,solid2,group,W-winW-2*PADDING,H-TITLE_H-RotatorH-22);
      infopanel.add("Center",boardbox);
      Rotator rotator= new Rotator(this,solid,W-winW-2*PADDING,RotatorH);
      new Thread(rotator).start(); 
      infopanel.add("South",rotator);
      scores = new Title(this,"",W-winW-2*PADDING,21);
      scores.font = new Font("TimesRoman",Font.ITALIC,18);
      RecordScore(0);
      infopanel.add("North",scores);
      add(infopanel);

      is.close();
      show();
    } catch (MalformedURLException e){
      System.out.println("MalformedURLException --config!");
    } catch (IOException e){
      System.out.println("IOException --config!");
    }

    purrr= this.getAudioClip(here,"purrr.au");

    kicker = new Thread(this);
    new Thread(kicker).start();
  }

  /* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - */
  public void RecordScore(int rotations){
    if (level1) scores.title = (String) "Rotations "+rotations+"/"+board.minRot;
    else scores.title = (String) "Rotations "+rotations+"/?";
    scores.centre();
    scores.repaint();
  }

  /* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - */
  public void run(){
    rotations=0;
    while(true){
      /* .. wait for things to change .. */
      playarea.thingsChange();  

      if (rotations != playarea.rotations){
        rotations = playarea.rotations;
        RecordScore(rotations);
      }
      if (board.nvert == board.NumCorrect() && !finished){
        finished = true;
        subtitle.setBackground(new Color(255,150,150));
        if (level1) 
          subtitle.title=(String) "Finished ("+rotations+"/"+board.minRot+")";
        else 
          subtitle.title=(String) "Finished ("+rotations+"/?)";
        purrr.play();
        subtitle.centre();
        subtitle.repaint();
      }
    }
  }

  /* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - */
  /* .. this is a hidden feature .. */
  public boolean keyDown (Event evt, int key){
    if (key == 'c'){
      /* .. print out the current board (for making new boards) .. */
      board.printCurrent(); 
      return true;
    }else return false;
  }

  /* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - */
  public boolean mouseDown(java.awt.Event evt, int x, int y) {
    if (y<TITLE_H){
      finished = false;
      rotations = 0;
      playarea.rotations =0;
      playarea.goToStart(true);
      board.Blank();  
      RecordScore(rotations);
      playarea.drawBackdrop();
      playarea.repaint();
      return true;
    }
    return false;
  }
}
